/****************************************************************************************/
/* 				  Datum Inc						*/			
/*			      San Jose Division						*/
/*			bc635PCI Time & Frequency Processor				*/
/*			Rev 1.0 March 15, 2000  SDarras          			*/
/****************************************************************************************/
/****************************************************************************************/
/* bc635pci.c is a C test program that demonstrates the operation of the different	*/
/* functions of the bc635PCI Time & Frequency Module using the stfp device driver.      */
/* The code is written in such a way that the user can cut and paste parts of the       */
/* program to their own custom application.						*/
/****************************************************************************************/
/****************************************************************************************/
/* To rebuild the stfp driver, use the following procedure:				*/
/*											*/
/* 1) Remove the old driver:								*/
/*		# rem_drv stfp								*/
/* 2) Compile the driver code (stfp.c):							*/
/*		# cc -D_KERNEL -c stfp.c						*/
/*		# ld -r -o stfp stfp.o							*/
/* 3) Copy the new driver to /kernel/drv						*/
/* 4) Add the new driver								*/
/*		# add_drv -i "pci12e2,4013" stfp					*/
/*		# devlinks								*/
/* 5) you may need to add "" to etc/driver_aliases file					*/
/*    if the line in etc/driver_aliases is: 						*/
/*		    stfp pci12e2,4013							*/
/*    change it to: stfp "pci12e2,4013" 						*/
/****************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <termio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <bc635pci_io.h>

/**************************************************************************************/
/* Global Variables and functions 
/**************************************************************************************/
int fd,tm_fmt;
struct getdata_t get;

int kbhit (void);
int WaitForKbhit(int,int);

/**************************************************************************************/
/* Description : main
/**************************************************************************************/
main ()
{    
	if ( (fd = open ("/dev/btfp0", O_RDWR)) < 0 ){				/* Open device */			
		printf("Error opening Device Driver ......... Exiting");
		_exit(0);
	}

	get.arg = GETDATA_TFORMAT;						/* Time Format */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        tm_fmt = (int)get.data.tformat;

   menu_main ();
   close(fd);
   _exit(1);									/* close any open files */
}

/**************************************************************************************/
/* Description : Read Time in Binary/Decimal
/**************************************************************************************/
pci_read_time ()
{
  struct stfp_time stm;			
  struct stfp_timeval tvTime;	
  int i;

	printf ("\n\n\n\n");
	if (tm_fmt == 0){  				/* Decimal Time Format*/
		do {
			if (!WaitForKbhit(0, 100)){
			ioctl (fd, RDTIMEREQ, &stm);				
			printf (" Julian Time: %03d %d %02d:%02d:%02d.%06d   Status: %x\n",
			stm.tm.tm_yday+1, stm.tm.tm_year+1900,
			stm.tm.tm_hour, stm.tm.tm_min, stm.tm.tm_sec, stm.usec, stm.status);
			}
		}while(!kbhit());
	} 
	else {							/* Binary Time Format */						
		do {
		    if (!WaitForKbhit(0, 100)){
			ioctl (fd, RDTIMETVREQ, &tvTime);
			printf ("Binary Time: %lu.%06ld status: %x     ", 
			tvTime.tv.tv_sec, tvTime.tv.tv_usec, tvTime.status);
			printf ("%s", ctime ((time_t *)&tvTime.tv.tv_sec));
			}
		}while(!kbhit());
	}
	fgetc (stdin);
	fflush (stdin);
}

/**************************************************************************************/
/* Description : Read Event Time 
/**************************************************************************************/
pci_read_event_time()
{		
  struct stfp_timeval tvTime;	
  unsigned long evtmaj, evtmin;

	printf ("\n\n"); /* Binary Time Format */
					
	do {
		ioctl (fd, RDEVENTTV, &tvTime);
		if ( tvTime.tv.tv_sec != evtmaj || tvTime.tv.tv_usec != evtmin){
			printf ("Event Time: %lu.%06ld status: %x     ", 
			tvTime.tv.tv_sec, tvTime.tv.tv_usec, tvTime.status);
			printf ("%s", ctime ((time_t *)&tvTime.tv.tv_sec));
		}
			evtmaj = tvTime.tv.tv_sec;
			evtmin = tvTime.tv.tv_usec;
		
	}while(!kbhit());

	fgetc (stdin);
	fflush (stdin);
}

/**************************************************************************************/
/* Description : Set Strobe Time
/**************************************************************************************/

pci_set_strobe ()
{
	int strbmode;
	struct stfp_time stm;
	struct stfp_timeval tvTime;	
	unsigned long tm_sec, tm_usec;
	int styear, stmon, stday, sthour, stmin, stsec;
	int	ioctl_arg = 0;

	printf ("\n\nSelect Strobe Mode:\n\n");
	printf (" 1. Major and Minor\n");
	printf (" 2. Minor Only\n");
	printf (" 0. Cancel\n\n");
	printf ("    Select: ");
	scanf  ("%d", &strbmode);

	if (strbmode == 1){
		ioctl_arg = STROBE_SECUS;
		ioctl (fd, CONTROLSTROBE, (void *)&ioctl_arg);
		
		printf("\n\nEnter Major Time:\n\nYYYY MM DD HH MM SS\n");
		scanf("%d %d %d %d %d %d", &styear, &stmon, &stday,
			&sthour, &stmin, &stsec);
			
			stm.tm.tm_year 	= styear - 1900;
			stm.tm.tm_mon	= stmon - 1;
			stm.tm.tm_mday	= stday;
			stm.tm.tm_hour 	= sthour;
			stm.tm.tm_min	= stmin;
			stm.tm.tm_sec	= stsec;
			tm_sec = mktime ((struct tm *)&stm);
			tvTime.tv.tv_sec = tm_sec;
			printf("\nEnter Minor time: ");
			scanf("%lu", &tm_sec);
			tvTime.tv.tv_usec =tm_usec;
			if (ioctl (fd, WRSTROBETV, &tvTime)<0)
				errin ();
	}
	else if (strbmode == 2){
		ioctl_arg = STROBE_USONLY;
		ioctl (fd, CONTROLSTROBE, (void *)&ioctl_arg);
		tvTime.tv.tv_sec = 0;
		printf("\nEnter minor time: ");
		scanf("%lu", &tm_usec);
		tvTime.tv.tv_usec =tm_usec;
		if (ioctl (fd, WRSTROBETV, &tvTime)<0)
			errin ();
	}
}

/**************************************************************************************/
/* Description: Set Control Register
/**************************************************************************************/
pci_set_control()
{
	unsigned long ctlreg;
	
	if (ioctl(fd, RDCONTROL, &ctlreg)<0)
	 errout();
	ctlreg &= 0xff;
	printf ("\n\nCurrent Control Register Value: 0x%02lx", ctlreg);
	printf ("\nEnter New CR Value (two Hex): ");
	scanf  ("%lx", &ctlreg);
	if (ioctl(fd, SETCONTROL, (void *)&ctlreg)<0)
	  errin();
}

/**************************************************************************************/
/* Description: Set Timing Mode
/**************************************************************************************/
pci_mode()
{
	int oper_mode = 0;
	int	ioctl_arg = 0;
	printf ("\n\nSelect Operational Mode:\n\n");
	printf (" 0. Time Code Mode\n");
	printf (" 1. Free Running Mode\n");
	printf (" 2. External 1PPS\n");
	printf (" 3. Real Time Clock Mode\n");
	printf (" 4. GPS Mode\n\n");
	printf ("    Select: ");
	scanf  ("%d", &oper_mode);

	switch (oper_mode){
		case 0: 
				ioctl_arg = MODE_TIMECODE;
					ioctl (fd, SELTIMINGMODE, (void *)&ioctl_arg);		/* TimeCode Mode */
				break;
		case 1:
				ioctl_arg = MODE_FREERUN;
				ioctl (fd, SELTIMINGMODE, (void *)&ioctl_arg);		/* Free Running Mode */
				break;
		case 2: 
				ioctl_arg = MODE_EXT1PPS;
				ioctl (fd, SELTIMINGMODE, (void *)&ioctl_arg);		/* 1PPS Mode */
				break;
		case 3: 
				ioctl_arg = MODE_RTC;
				ioctl (fd, SELTIMINGMODE, (void *)&ioctl_arg);			/* RTC Mode */
				break;
		case 4: 
				ioctl_arg = MODE_GPS;
				ioctl (fd, SELTIMINGMODE, (void *)&ioctl_arg);			/* GPS Mode */
				break;
		default:usererr();
				break;
	}
}

/**************************************************************************************/
/* Description : Set Time Code Format & Type
/**************************************************************************************/
pci_time_code ()
{
	int tc_fmt;
	int tc_mod;
	int	ioctl_arg;

	printf ("\n\nSelect Time Code Format:\n\n");
	printf (" 1. IRIG A\n");
	printf (" 2. IRIG B\n");
	printf (" 3. IEEE 1344\n");
	printf (" 4. NASA 36\n\n");
	printf ("    Select: ");
	scanf  ("%d", &tc_fmt);
	switch ( tc_fmt )
	{
		case 1: ioctl_arg = TC_IRIGA;
				ioctl (fd, SELTCFORMAT, (void *)&ioctl_arg);			/* IRIG A time code */
				break;
		case 2: ioctl_arg = TC_IRIGB;
				ioctl (fd, SELTCFORMAT, (void *)&ioctl_arg);			/* IRIG B time code */
				break;
		case 3: ioctl_arg = TC_IEEE;
				ioctl (fd, SELTCFORMAT, (void *)&ioctl_arg);			/* IEEE time code */
				break;
		case 4: ioctl_arg = TC_NASA ;
				ioctl (fd, SELTCFORMAT, (void *)&ioctl_arg);			/* NASA 36 time code */
				break;
		default:usererr();
				break;
	}

	printf ("\n\nSelect Time Code Modulation:\n\n");
	printf (" 1. Modulated\n");
	printf (" 2. DC Level Shift\n\n");
	printf ("    Select: ");
	scanf  ("%d", &tc_mod);
	switch ( tc_mod )
	{
		case 1: ioctl_arg = MOD_AM;
				ioctl (fd, SELTCMOD, (void *)&ioctl_arg);			/* Modulated */
				break;
		case 2: ioctl_arg = MOD_DCLS;
				ioctl (fd, SELTCMOD, (void *)&ioctl_arg);			/* DCLS */
				break;
		default:usererr();
				break;
	}
}

/**************************************************************************************/
/* Description : Set Output Frequency
/**************************************************************************************/
pci_out_freq ()
{
    int out_freq = 3;
	int	ioctl_arg = 0;

	printf ("\n\nSelect Output Frequency:\n\n");
	printf (" 1. 1MHz\n 2. 5MHz\n 3. 10MHz\n\n");
	printf (" Select: ");
	scanf  ("%d", &out_freq);
	switch ( out_freq )
	{
		case 1: ioctl_arg = FREQ_1MHZ;
				ioctl (fd, SELFREQUENCYOUT, (void *)&ioctl_arg);			/* 1 MHz */
				break;
		case 2: ioctl_arg = FREQ_5MHZ;
				ioctl (fd, SELFREQUENCYOUT, (void *)&ioctl_arg);			/* 5 MHz */
				break;
		case 3: ioctl_arg = FREQ_10MHZ;
				ioctl (fd, SELFREQUENCYOUT, (void *)&ioctl_arg);		/* 10 MHz */
				break;
		default:usererr();
				break;
	}
}

/**************************************************************************************/
/* Description : Set Time Format
/**************************************************************************************/
pci_time_format()
{
	int time_format = 0;
	int	ioctl_arg = 0;

	printf ("\n\nSelect Time Format\n  0. Decimal\n  1. Binary\n\nSelect: ");
	scanf ("%d", &time_format);
	
	switch ( time_format )
	{
		case 0: ioctl_arg = TIME_DECIMAL;
				ioctl (fd, SELTIMEFORMAT, (void *)&ioctl_arg);	/* decimal time format */
					tm_fmt = 0;
				break;
		case 1: ioctl_arg = TIME_BINARY;
				ioctl (fd, SELTIMEFORMAT, (void *)&ioctl_arg);		/* Binary time format */
					tm_fmt = 1;
				break;				
		default:usererr();
				break;
	}
}
	
/**************************************************************************************/
/* Description : Set Periodic (Heartbeat) Output
/**************************************************************************************/
pci_heartbeat ()
{
	struct periodic sper;
	int hrt_mode;
	int counter1 = 10; 
	int counter2 = 10;
	sper.sync1pps = PERIODIC_SYNC;

	printf ("\n\nHeartbeat Counters and Mode:\n\n");
	printf (" 1. Synchronous mode\n");
	printf (" 2. Asynchronous mode\n\n");
	printf (" Select: ");
	scanf  ("%i", &hrt_mode);
	printf ("\n Enter decimal value for counetr1: ");
	scanf  ("%d", &counter1);
	printf ("\n Enter decimal value for counetr2: ");
	scanf  ("%d", &counter2);

	if (hrt_mode == 2)
		sper.sync1pps = PERIODIC_NOSYNC;
	if (counter1 >= 2 && counter1 <= 65535)
		sper.n1 = counter1;
	if (counter2 >= 2 && counter2 <= 65535)
		sper.n2 = counter2;
	
	if (ioctl (fd, SETPERIODIC, &sper) < 0){
		printf("\nError setting Heartbeat Counters!!!");
		nxt();
	}
}

/**************************************************************************************/
/* Description : Set Year
/**************************************************************************************/
pci_set_year ()
{
	int year = 2000;

	printf ("\n\nEnter Year (1970 to 2050): ");
	scanf  ("%d", &year);
	if (year > 1970 && year < 2050){
		if (ioctl (fd, SETYEAR, (void *)&year) < 0){			/* Set year */
			errin();
			nxt();
		}
	}
	else
		usererr();

}

/**************************************************************************************/
/* Description : Set Time
/**************************************************************************************/

pci_set_time ()
{
	struct stfp_time stm;		
	struct stfp_timeval tvTime;	
	unsigned long tm_sec;
	struct stfp_dec_tm dec;
	int styear, stmon, stday, sthour, stmin, stsec;


	/* Decimal Time Format */
	if (tm_fmt == 0)
	{
		printf("\n\nEnter Time:\n\nYYYY DDD HH MM SS\n");
		scanf("%d %d %d %d %d", &styear, &stday,
			&sthour, &stmin, &stsec);
			
			dec.dec_year 	= styear;
			dec.dec_day	= stday;
			dec.dec_hour 	= sthour;
			dec.dec_min	= stmin;
			dec.dec_sec	= stsec;
		
		if ( ioctl (fd, SETDECTIME, (void *)&dec) < 0 )
			printf("\nError Setting Time!!");
	}
	else	/* Binary time format */
	{
		printf("\n\nEnter Time:\n\nYYYY MM DD HH MM SS\n");
		scanf("%d %d %d %d %d %d", &styear, &stmon, &stday,
			&sthour, &stmin, &stsec);
			
			stm.tm.tm_year 	= styear - 1900;
			stm.tm.tm_mon	= stmon - 1;
			stm.tm.tm_mday	= stday;
			stm.tm.tm_hour 	= sthour;
			stm.tm.tm_min	= stmin;
			stm.tm.tm_sec	= stsec;
			tm_sec = mktime ((struct tm *)&stm);
		
		if ( ioctl (fd, SETTIME, (void *)&tm_sec) < 0 )
			printf("\nError Setting Time!!");
	}
}

/**************************************************************************************/
/* Description : Set Propogation Delay
/**************************************************************************************/
pci_set_prop_delay ()
{
	long prop_delay = 0;

	printf("\n\nEnter Propogation Delay: ");
	scanf("%ld", &prop_delay);

	if ( ioctl (fd, SETTIMINGOFFSET, (void *)&prop_delay) < 0 ){
		errin();
		nxt();
	}
}

/**************************************************************************************/
/* Description : Software Rest
/**************************************************************************************/
pci_sw_rest ()
{
	int yesno;
	int	ioctl_arg = 0;
	printf("\n\nReset the board?\n 0. No\n 1.Yes\n\nSelect: ");
	scanf("%d", &yesno);

	if ( yesno == 1 ){
		ioctl_arg = 1;
		if (ioctl (fd, SOFTWARERESET, (void *)&ioctl_arg) < 0){
			printf("\nError reseting the card!!");
			nxt();
		}
	}
}

/**************************************************************************************/
/* Description : Set TimeCode Output Format
/**************************************************************************************/
pci_tc_out_format()
{
	int tc_out = 0;
	int	ioctl_arg = 0;
	printf ("\n\nSelect Time Code Output Format\n  0. IRIG B\n  1. IEEE 1344\n\nSelect: ");
	scanf ("%d", &tc_out);
	
	switch ( tc_out )
	{
		case 0: ioctl_arg = TC_IRIGB;
				ioctl (fd, SETTCOUTFMT, (void *)&ioctl_arg);	
				break;
		case 1: ioctl_arg = TC_IEEE;
				ioctl (fd, SETTCOUTFMT, (void *)&ioctl_arg);		
				break;				
		default:usererr();
				break;
	}
}

/**************************************************************************************/
/* Description : Set Clock Source
/**************************************************************************************/
pci_set_clock_src()
{
	int clk_src = 0;
	int	ioctl_arg = 0;

	printf ("\n\nSelect Clock Source\n  0. Internal\n  1. External\n\nSelect: ");
	scanf ("%d", &clk_src);
	
	switch ( clk_src )
	{
		case 0: ioctl_arg = CLOCK_INTERNAL;
				ioctl (fd, CONTROLTIMEBASE, (void *)&ioctl_arg);	
				break;
		case 1: ioctl_arg = CLOCK_EXTERNAL;
				ioctl (fd, CONTROLTIMEBASE, (void *)&ioctl_arg);		
				break;				
		default:usererr();
				break;
	}
}

/**************************************************************************************/
/* Description : Set Generator Time Offset
/**************************************************************************************/
pci_set_gen_off ()
{
	struct tcgenoffset gen;
	short	gentc;			
	int	gentcflg;

	printf ("\n\nEnter Generator Time Offset (-16 to 16): ");
	scanf  ("%hd", &gentc);

	printf ("\nAdd Half Hour?\n 0: No\n 1: Yes\nSelect: ");
	scanf ("%d", &gentcflg);

	gen.genoff = gentc;
	gen.genflg = (u_char)gentcflg;
	
	if ( ioctl (fd, SETGENTMOFFSET, &gen) < 0 ){
		printf("\nError setting generator time offset");
		nxt();
	}

}

/**************************************************************************************/
/* Description : Set Local Time Offset
/**************************************************************************************/
pci_set_loc_off ()
{
	struct	loctmoffset loc;
	short	localoff;			
	int		localflg;

	printf ("\n\nEnter Local Time Offset (-16 to 16): ");
	scanf  ("%hd", &localoff);

	printf ("\nAdd Half Hour?\n 0: No\n 1: Yes\nSelect: ");
	scanf ("%d", &localflg);

	loc.locoff = localoff;
	loc.locflg = (u_char)localflg;
	
	if ( ioctl (fd, SETLOCTMOFFSET, &loc) < 0 ){
		printf("\nError setting local time offset");
		nxt();
	}

}
  
/**************************************************************************************/
/* Description : Program Leap Seconds
/**************************************************************************************/
pci_set_leap_sec ()
{
	struct	leapseconds leap;
	long	leaptm;			
	int		leapflag;

	printf ("\n\nEnter Leap Event Time in Seconds (Unix Time): ");
	scanf  ("%ld", &leaptm);

	printf ("\nLeap Second Flag?\n  0: Disable\n  1: Insertion\n -1: Deletion\nSelect: ");
	scanf ("%d", &leapflag);

	leap.leap_tm = leaptm;
	leap.ls_flag = (u_char)leapflag;
	
	if ( ioctl (fd, SETLEAPSECEVENT, &leap) < 0 ){
		printf("\nError setting leap event time");
		nxt();
	}

}      
        
/**************************************************************************************/
/* Description : Control Jam-Sync
/**************************************************************************************/
pci_ctl_jam_sync()
{
	int jam_sync = 1;
	int	ioctl_arg = 0;
	printf ("\n\nControl Jam-Sync\n 0. Disable Jam-Sync\n 1. Enable Jam-Sync\n\nSelect: ");
	scanf ("%d", &jam_sync);
	
	switch ( jam_sync )
	{
		case 0: ioctl_arg = JAMSYNC_DISABLE;
				ioctl (fd, CONTROLTIMEBASE, (void *)&ioctl_arg);	
				break;
		case 1: ioctl_arg = JAMSYNC_ENABLE;
				ioctl (fd, CONTROLTIMEBASE, (void *)&ioctl_arg);		
				break;				
		default:usererr();
				break;
	}
}

/**************************************************************************************/
/* Description : Force Jam-Sync
/**************************************************************************************/
pci_frc_jam_sync()
{
	int jam_sync = 0;
	int	ioctl_arg = 0;

	printf ("\n\nForce Jam-Sync?\n 0. No\n 1. Yes\n\nSelect: ");
	scanf ("%d", &jam_sync);
	
	if ( jam_sync ){
		ioctl_arg = FORCE_JAMSYNC;
		ioctl (fd, CONTROLTIMEBASE, (void *)&ioctl_arg);	
		nxt();
	}
}

/**************************************************************************************/
/* Description : Load D/A Converter
/**************************************************************************************/
pci_set_da_con()
{
	u_short da_con = 0;
	
	printf ("\n\nEnter D/A value (0 to 65535): ");
	scanf ("%d", &da_con);
	
	if ( ioctl (fd, SETDAC, (void *)&da_con) < 0 ){
		printf("\nError setting D/A converter!");		
		nxt();
	}
}

/**************************************************************************************/
/* Description : Set Disciplining Gain
/**************************************************************************************/
pci_set_gain()
{
	short dis_gain = 0;
	
	printf ("\n\nEnter D/A value (-32768 to 32767): ");
	scanf ("%hd", &dis_gain);
	
	if ( ioctl (fd, SETDISCGAIN, (void *)&dis_gain) < 0 ){
		printf("\nError setting disciplining gain");		
		nxt();
	}
}

/**************************************************************************************/
/* Description : Sync RTC to External Time Data
/**************************************************************************************/
pci_sync_rtc()
{
	int sync_rtc = 0;
	
	printf ("\n\nSync RTC to External Time Data?\n 0. No\n 1. Yes\n\nSelect: ");
	scanf ("%d", &sync_rtc);
	
	if ( sync_rtc ){
		if ( ioctl (fd, SYNCRTC, NULL) < 0 ){
			errin();
			nxt();
		}
	}
}

/**************************************************************************************/
/* Description : Disconnect Battery to RTC Chip
/**************************************************************************************/
pci_dis_rtc()
{
	int dis_rtc = 0;
	
	printf ("\n\nDisconnect Battery to RTC Chip?\n 0. No\n 1. Yes\n\nSelect: ");
	scanf ("%d", &dis_rtc);
	
	if ( dis_rtc ){
		if ( ioctl (fd, DISCBATT, NULL) < 0 ){
			errin();
			nxt();
		}
	}
}

/**************************************************************************************/
/* Description : Set Clock Value
/**************************************************************************************/
pci_set_clk_val()
{
	long clk_val = 0;
	
	printf ("\n\nEnter Clock Value: ");
	scanf ("%ld", &clk_val);
	
	if ( ioctl (fd, SETCLKVAL, (void *)&clk_val) < 0 ){
		printf("\nError setting clock value");
		nxt();
	}
}


/**************************************************************************************/
/* Description : Set GPS Time Format
/**************************************************************************************/
pci_set_gps_tm_fmt()
{
	int gps_tm_fmt = 0;
	int	ioctl_arg = 0;

	printf ("\n\nGPS Time Format\n 0. UTC Time\n 1. GPS Time\n\nSelect: ");
	scanf ("%d", &gps_tm_fmt);
	
	switch ( gps_tm_fmt )
	{
		case 0: ioctl_arg = UTC_FMT;
				ioctl (fd, SETGPSTMFMT, (void *)&ioctl_arg);	
				break;
		case 1: ioctl_arg = GPS_FMT;
				ioctl (fd, SETGPSTMFMT, (void *)&ioctl_arg);		
				break;				
		default:usererr();
				break;
	}
}

/**************************************************************************************/
/* Description : Set GPS Mode Flag
/**************************************************************************************/
pci_set_gps_mode_flg()
{
	int gps_mode_flg = 1;
	int	ioctl_arg = 0;

	printf ("\n\nGPS Mode Flag\n 0. Disable\n 1. Enable\n\nSelect: ");
	scanf ("%d", &gps_mode_flg);
	
	switch ( gps_mode_flg )
	{
		case 0: ioctl_arg = GPS_FLG_DIS;
				ioctl (fd, SETGPSTMFMT, (void *)&ioctl_arg);	
				break;
		case 1: ioctl_arg = GPS_FLG_ENA;
				ioctl (fd, SETGPSTMFMT, (void *)&ioctl_arg);		
				break;				
		default:usererr();
				break;
	}
}

/**************************************************************************************/
/* Description : Set Local Time Offset Flag
/**************************************************************************************/
pci_set_local_off_flg()
{
	int local_off_flg = 1;
	int	ioctl_arg = 0;

	printf ("\n\nLocal Time Offset Flag\n 0. Disable\n 1. Enable\n\nSelect: ");
	scanf ("%d", &local_off_flg);
	
	switch ( local_off_flg )
	{
		case 0: ioctl_arg = LOC_OFF_DIS;
				ioctl (fd, SETLOCTMFLG, (void *)&ioctl_arg);	
				break;
		case 1: ioctl_arg = LOC_OFF_ENA;
				ioctl (fd, SETLOCTMFLG, (void *)&ioctl_arg);		
				break;				
		default:usererr();
				break;
	}
}

/**************************************************************************************/
/* Description : Set Year Auto Increment Flag
/**************************************************************************************/
pci_set_yr_auto_inc_flg()
{
	int yr_auto_flg = 1;
	int	ioctl_arg = 0;
	printf ("\n\nYear Auto Increment Flag\n 0. Disable\n 1. Enable\n\nSelect: ");
	scanf ("%d", &yr_auto_flg);
	
	switch ( yr_auto_flg )
	{
		case 0: ioctl_arg = YR_INC_DIS;
				ioctl (fd, SETYRINCFLG, (void *)&ioctl_arg);	
				break;
		case 1: ioctl_arg = YR_INC_ENA;
				ioctl (fd, SETYRINCFLG, (void *)&ioctl_arg);		
				break;				
		default:usererr();
				break;
	}
}

/**************************************************************************************/
/* Description : Request Time Settings
/**************************************************************************************/
pci_req_time_settings ()
{
	const char smode[7][10] = {"Time Code","Free Run","Ext 1PPS","RTC","Reserved","Reserved","GPS"};
	const char stm_fmt[2][10] = {"Decimal","Binary"};
	u_char tc_fmt, tc_mod, mode, tc_out;
	int year;
	
	get.arg = GETDATA_MODE;		/* Timing Mode */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        mode = (int)get.data.tmode;

	get.arg = GETDATA_TFORMAT;	/* Time Format */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        tm_fmt = (int)get.data.tformat;

	get.arg = GETDATA_YEAR ;		/* Year */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        year = (int)get.data.year;

	get.arg = GETDATA_TCFMT;		/* Time Code Format */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        tc_fmt = (u_char)get.data.tcformat;

	get.arg = GETDATA_TCMOD;		/* Time Code Modulation */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        tc_mod = (u_char)get.data.tcmod;

	get.arg = GETDATA_TCOUTFMT;		/* Time Code Output Format */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        tc_out = (u_char)get.data.tcoutfmt;

	printf("\n\nTime Settings:\n");
	printf("\n Mode\t\t\t: %s",smode[mode]);
	printf("\n Time Format\t\t: %s",stm_fmt[tm_fmt]);
	printf("\n Year\t\t\t: %d",year);
	
	switch (tc_fmt){
	case 'A': 
	case 'B': printf("\n Time Code\t\t: IRIG %c",tc_fmt);
		break;
	case 'N': printf("\n Time Code\t\t: %cASA 36",tc_fmt);
		break;
	case 'I': printf("\n Time Code\t\t: %cEEE 1344",tc_fmt);
		break;
	default: printf("\n Time Code\t\t: Unknown");
		break;
	}

	switch ( tc_mod ){
	case 'M': printf("\n Code Modulation\t: AM");
		break;
	case 'D': printf("\n Code Modulation\t: DC");
		break;
	default: printf("\n Code Modulation\t: Unknown");
		break;
	}

	switch ( tc_out ){
	case 'B': printf("\n TimeCode Out\t\t: IRIG B");
		break;
	case 'I': printf("\n TimeCode Out\t\t: IEEE 1344");
		break;
	default: printf("\n TimeCode Out\t\t: Unknown");
		break;
	}	

	nxt();
}

/**************************************************************************************/
/* Description : Request Clock Settings
/**************************************************************************************/
pci_req_clock_settings ()
{
	int dac_val, jam_ctl, bat_st, clk_val, dis_ctl, ph_ctl;
	int phase_st, dis_gain;
	const char jam[2][8]={"Disable","Enable"};
	const char batt[2][8]={"Failed","OK"};
	float filter;
	u_char clk_scr;
	
	get.arg = GETDATA_CLKSRC;		/* Clock Source */	
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        clk_scr = (u_char)get.data.clksrc;

	get.arg = GETDATA_DAVAL;		/* DAC Value */		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        dac_val = (int)get.data.davalue;
    
	get.arg = GETDATA_JAMSC;		/* Jam Control */	
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        jam_ctl = get.data.jsctl & 0x01;

	get.arg = GETDATA_BATTSTAT;		/* Batt Status */		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        bat_st = (get.data.battstat & 0x10) >> 4;
	
	get.arg = GETDATA_CLKVAL;		/* ClockValue */		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        clk_val = (int)get.data.clkval;

	get.arg = GETDATA_OSCCTL;		/* Oscillator Disciplining Control */		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) {
        dis_ctl = get.data.ocsctl.discctl;
		ph_ctl = get.data.ocsctl.phasectl;
		phase_st = (int)get.data.ocsctl.phasestep;
		dis_gain = (int)get.data.ocsctl.gain;
		filter = (float)get.data.ocsctl.con;
	}

	printf("\n\nClock Settings:\n");
	
	if (clk_scr == 'E')
		printf("\n Oscillator  \t: External");
	else
		printf("\n Oscillator  \t: Internal");
	
	printf("\n DAC Value\t: %d",dac_val);
	printf("\n Jam Control\t: %s",jam[jam_ctl]);
	printf("\n Battery Status\t: %s",batt[bat_st]);
	printf("\n Clock Value\t: %d",clk_val);
	printf("\n Disc Control\t: %s",jam[dis_ctl]);
	printf("\n Phase Control\t: %s",jam[ph_ctl]);
	printf("\n Phase Step\t: %d",phase_st);
	printf("\n Disc Gain\t: %d",dis_gain);
	printf("\n Filter Const\t: %f",filter);

	nxt();
}

/**************************************************************************************/
/* Description : Request Offsets Settings
/**************************************************************************************/
pci_req_offsets_settings ()
{
	float loc_off, gen_off;
	int prop_off, loc_off_flg, gen_off_flg;
	
	get.arg = GETDATA_LOCTMOFF;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ){
        loc_off = (float)get.data.locoff.locoff;
		loc_off_flg = (int)get.data.locoff.locflg;
	}
	if ( loc_off_flg )
		loc_off += 0.5;
	
	get.arg = GETDATA_TCGENOFF;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ){
        gen_off = (float)get.data.genoff.genoff;
		gen_off_flg = (int)get.data.genoff.genflg;
	}
	if ( gen_off_flg )
		gen_off += 0.5;
	
	get.arg = GETDATA_TOFF;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        prop_off = (int)get.data.offset;

	printf("\n\nOffsets Settings:\n");
	printf("\n Local Offset     \t: %.1f",loc_off);
	printf("\n Generator Offset \t: %.1f",gen_off);
	printf("\n Propogation Delay\t: %d",prop_off);

	nxt();
}

/**************************************************************************************/
/* Description : Request UTC Information 
/**************************************************************************************/
pci_req_utc_info ()
{
	int fmt, leap_sec, leap_sec_flg;
	const char gps_fmt[2][4]={"UTC","GPS"};
	unsigned int evttm;
	
	get.arg = GETDATA_UTCINFO;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ){ 
		fmt = get.data.utc.enable & 0x01;
		leap_sec = get.data.utc.leapsec;
		leap_sec_flg = get.data.utc.leapevt;
		evttm = get.data.utc.evttime;
		
		printf("\n\nUTC Information\n");
		printf("\n GPS Time Format\t: %s\n Leap Seconds\t\t: %d", gps_fmt[fmt], leap_sec);
		
		switch (leap_sec_flg){
		case -1: printf("\n Leap Second Flag\t: Deletion");
			break;
		case 0:  printf("\n Leap Second Flag\t: Disable");
			break;
		case 1:  printf("\n Leap Second Flag\t: Addition");
			break;
		default: printf("\n Leap Second Flag\t: Unknown");
			break;
		}
		printf("\n Leap Event Time\t: %lu", evttm);
	}
	else
		printf("\nError retrieving data!!");

	nxt();
}

/**************************************************************************************/
/* Description : Request Model, Serial Number, Assembly Number and Hardware FAB 
/**************************************************************************************/
pci_req_assembly ()
{
	u_char t1,t2,t3,t4,t5,t6,t7,t8;
	u_short assembly, hw_fab;
	unsigned int ser_num;

	printf("\n\nModel Information\n");

	get.arg = GETDATA_TFPMODEL;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ){ 
		t1 = get.data.tfpm.tfp1;
		t2 = get.data.tfpm.tfp2;
		t3 = get.data.tfpm.tfp3;
		t4 = get.data.tfpm.tfp4;
		t5 = get.data.tfpm.tfp5;
		t6 = get.data.tfpm.tfp6;
		t7 = get.data.tfpm.tfp7;
		t8 = get.data.tfpm.tfp8;
		printf("\n Model\t\t: %c%c%c%c%c%c%c%c",
					t1,t2,t3,t4,t5,t6,t7,t8);
	}

	get.arg = GETDATA_SERIAL;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ){ 
		ser_num = get.data.serialnum;
		printf("\n Serial Number\t: %lx", ser_num);
	}

	get.arg = GETDATA_ASSEMB;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ){ 
		assembly = (u_short)get.data.assemb;
		printf("\n Assembly Num\t: %hu", assembly);
        }

	get.arg = GETDATA_HWFAB;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ){ 
		hw_fab = (u_short)get.data.hwfab;
		printf("\n Hardware FAB\t: %hu", hw_fab);
        }
	else
		printf("\nError retrieving data!!");

	nxt();
}


/**************************************************************************************/
/* Description : Request Firmware Version
/**************************************************************************************/
pci_req_fw_ver ()
{
	int fwmaj, fwmin, fwmon, fwday, fwyr;
	u_char fwb01, fwb02,fwb03,fwb04,fwb05,fwb06,fwb07,fwb08,fwb09,fwb10,fwb11;
 
    get.arg = GETDATA_DTFW;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ){ 
        fwb01 = (u_char)get.data.dtfw.dt01;
        fwb02 = (u_char)get.data.dtfw.dt02;
        fwb03 = (u_char)get.data.dtfw.dt03;
        fwb04 = (u_char)get.data.dtfw.dt04;
        fwb05  = (u_char)get.data.dtfw.dt05;
        fwb06 = (u_char)get.data.dtfw.dt06;
        fwb07 = (u_char)get.data.dtfw.dt07;
        fwb08 = (u_char)get.data.dtfw.dt08;
        fwb09 = (u_char)get.data.dtfw.dt09;
        fwb10  = (u_char)get.data.dtfw.dt10;
        fwb11  = (u_char)get.data.dtfw.dt11;
        printf("\n\nFirmware Information\n");
        printf("\n\n Number\t\t: %c%c%c%c%c%c%c%c%c%c%c", 
        fwb01,fwb02,fwb03,fwb04,fwb05,fwb06,fwb07,fwb08,fwb09,fwb10,fwb11);
	}

	get.arg = GETDATA_FWVER;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ){ 
        fwmaj = (u_char)get.data.version.vmajor;
        fwmin = (u_char)get.data.version.vminor;
        fwmon = (u_char)get.data.version.month;
        fwday = (u_char)get.data.version.day;
        fwyr  = (u_short)get.data.version.year;
		printf("\n Version\t: %d.%d\n Date\t\t: %d/%d/%d", 
        fwmaj, fwmin, fwmon,fwday,fwyr);
        }
	else
		printf("\nError retrieving firmware data!!");
	
	nxt();
}

/**************************************************************************************/
/* Description : Advanced Menu
/**************************************************************************************/
advanced_main ()
{
	int adv_menu;

	printf ("\n\n\n\nAdvanced Menu\n\n");
	printf (" 1. Control Jam-Sync\n");
	printf (" 2. Force Jam-Sync\n");
	printf (" 3. Set Clock Value\n");
	printf (" 4. Load D/A Converter\n");
	printf (" 5. Set Disciplining Gain\n");
	printf (" 6. Disconnect Battery to RTC Chip\n");
	printf (" 0. Back to main menu \n\n");
	printf ("    Select: ");
	scanf ("%d", &adv_menu);

	switch (adv_menu){
	case 1: pci_ctl_jam_sync();
		break;
	case 2: pci_frc_jam_sync();
		break;
	case 3: pci_set_clk_val();
		break;
	case 4: pci_set_da_con();
		break;
	case 5: pci_set_gain();
		break;
	case 6: pci_dis_rtc();
		break;
	case 0:
		break;
	default: usererr();
		break;
	}
}

/**************************************************************************************/
/* Description : Main Menu
/**************************************************************************************/
menu_main ()
{
	int menu;
	do
	{
		printf ("\n                Datum Inc. San Jose Division\n");
		printf ("              bc635PCI Time & Frequency Processor\n");
		printf (" 1. Read Current Time               16. Set Generator Time Offset\n");
		printf (" 2. Read Event Time                 17. Set Propogation Delay\n"); 
		printf (" 3. Set Current Time                18. Set Local Time Offset Flag\n");
		printf (" 4. Set Current Year                19. Set Year Auto Increment Flag\n");
		printf (" 5. Set Strobe Time                 20. Sync RTC to External Time Data\n");
		printf (" 6. Program Control Register        21. Set GPS Time Format\n"); 
		printf (" 7. Program Leap Event Seconds      22. Set GPS Mode Flag\n");
		printf (" 8. Select Time Format              23. Software Reset\n");
		printf (" 9. Select Operational Mode         24. Request Time Settings\n");
		printf ("10. Select Decoding TimeCode Format 25. Request Clock Settings\n");
		printf ("11. Select TimeCode Output Format   26. Request Offsets Settings\n");
		printf ("12. Select Clock Source             27. Request UTC Information\n");
		printf ("13. Select Output Frequency         28. Request Model Information\n");
		printf ("14. Program Heartbeat counters      29. Request Firmware Version\n");
		printf ("15. Set Local Time Offset           30. Advanced Menu\n");
		printf (" 0. Exit the Program.................................................\n");
		printf ("    Select: ");
		scanf ("%d", &menu);
    
		switch(menu) 
		{
			case 1 : pci_read_time();
					break;
			case 2 : pci_read_event_time();
					break;
			case 3 : pci_set_time ();
					break;
			case 4 : pci_set_year();
					break;
			case 5 : pci_set_strobe();
					break;
			case 6 : pci_set_control();
					break;
			case 7 : pci_set_leap_sec ();
					break;
			case 8 : pci_time_format();
					break;
			case 9 : pci_mode();
					break;
			case 10 : pci_time_code();
					break;
			case 11 : pci_tc_out_format(); 
					break;
			case 12 : pci_set_clock_src();
					break;
			case 13 : pci_out_freq(); 
					break;
			case 14: pci_heartbeat();
					break;
			case 15: pci_set_loc_off ();
					break;
			case 16: pci_set_gen_off ();
					break;
			case 17: pci_set_prop_delay ();
					break;
			case 18 : pci_set_local_off_flg();
					break;
			case 19 : pci_set_yr_auto_inc_flg();
					break;
			case 20 : pci_sync_rtc();
					break;
			case 21 : pci_set_gps_tm_fmt();
					break;
			case 22 : pci_set_gps_mode_flg();
					break;
			case 23 : pci_sw_rest ();
					break;
			case 24 : pci_req_time_settings();
					break;
			case 25 : pci_req_clock_settings ();
					break;
			case 26 : pci_req_offsets_settings ();
					break;
			case 27 : pci_req_utc_info ();
					break;
			case 28: pci_req_assembly ();
					break;
			case 29: pci_req_fw_ver ();
					break;
			case 30: advanced_main ();
					break;
			case 0  : 
			default : menu = 0;
					  printf("\n\nexiting..............................\n\n");
					  break;
        	}
	}while (menu);
}

/**************************************************************************************/
/* Description :  polls stdin to see if there is any input availible
/**************************************************************************************/
int kbhit ()
{
   fd_set rfds;
   struct timeval tv;
   FD_ZERO (&rfds);
   FD_SET (0, &rfds);
   tv.tv_sec  = 0;
   tv.tv_usec = 0;
   select (1, &rfds, NULL, NULL, &tv);
   if (FD_ISSET(0, &rfds))
      return 1;
   return 0;
}

/**************************************************************************************/
/* Description :  waits for selected time for any input to come into stdin then exits
/* a value of -1 for seconds means wait forever
/**************************************************************************************/
int WaitForKbhit(int seconds, int mseconds)
{
   fd_set rfds;
   struct timeval tv;
   while (1)
   {
      FD_ZERO (&rfds);
      FD_SET (0, &rfds);
      tv.tv_sec  = seconds == -1 ? 60 : seconds;
      tv.tv_usec = mseconds;
      select (1, &rfds, NULL, NULL, &tv);
      if (FD_ISSET(0, &rfds))
         return 1;
      if (seconds != -1)
         break;
   }
   return 0;
}

/**************************************************************************************/
/* Description : Error
/**************************************************************************************/
usererr ()
{
	printf ("\nError: Unrecognized Selection!!!\n");
	getchar();
	printf("\n\n>>>>>>>>>>>>>>>>  Press Enter to continue  <<<<<<<<<<<<<<<<<\n");
	getchar(); 
}
errout ()
{
	printf("\n\nError: getting data!!!!!!!!!!!!!!!!!!!!!!\n\n");
}
errin ()
{
	printf("\n\nError: sending data!!!!!!!!!!!!!!!!!!!!!!\n\n");
}

nxt ()
{
	getchar();
	printf("\n\n>>>>>>>>>>>>>>>>  Press Enter to continue  <<<<<<<<<<<<<<<<<\n");
	getchar(); 
}
